@@ -1,3 +1,4 @@
+import collections
 import contextlib
 import glob
 import inspect
@@ -182,6 +183,7 @@ class _inject:
         def f(*args, **kwargs):
             kwargs[self.attname] = require(self.injected)
             return obj(*args, **kwargs)
+        self._update_signature(f)
         return f
 
     def _decorate_coroutinefunction(self, obj):
@@ -193,8 +195,19 @@ class _inject:
             async with _ensure_async_context(dep, self.scope) as ctx:
                 kwargs[self.attname] = ctx
                 return await obj(*args, **kwargs)
+        self._update_signature(f)
         return f
 
+    def _update_signature(self, f):
+        sig = inspect.signature(f)
+        parameters = list(sig.parameters.items())
+        f.__annotations__ = collections.OrderedDict(
+            [(n, p) for n, p in parameters if n != self.attname]
+        )
+        f.__signature__ = sig.replace(
+            parameters=list(f.__annotations__.values())
+        )
+
 
 @contextlib.asynccontextmanager
 async def _ensure_async_context(dep, scope):
